import os
import re
import importlib
# 
def deleteFistDotte(string:str)-> str:
    if string.startswith('.'):
        return re.sub(r'^.', '', string)
    else:
        return string
    
def dynamicRoute(route_in:str)-> str:

    # Remplacer [id] par {id}
    route_out = re.sub(r"\[([^\]]+)\]", r"{\1}",route_in)
    # Remplacer {_slug} par {slug:path} pour capturer plusieurs segments
    route_out = re.sub(r"\{_([^\}]+)\}", r"{\1}:path", route_out)

    return route_out

def convertPathToModulePath(path:str)->str:
    return re.sub(r"\\|/", ".", path)

def importModule(path:str):
    try:
        module =importlib.import_module(path) 
    except ModuleNotFoundError as e:
        print(f"Error importing module '{path}': {e}")
    return module




    """
    Trouve le fichier page.html le plus proche dans le dossier courant ou ses parents.
    
    Returns:
        str or None: Le chemin absolu vers page.html si trouvé, None sinon.
    """
    try:
        # Obtenir le chemin absolu du dossier contenant le script actuel
        current_dir = os.path.dirname(os.path.abspath(__file__))
        
        # D'abord chercher dans le dossier courant
        page_path = os.path.join(current_dir, 'page.html')
        if os.path.isfile(page_path):
            return page_path
        
        # Si pas trouvé, remonter jusqu'au dossier 'app'
        while True:
            # Vérifier si on est dans le dossier 'app'
            if os.path.basename(current_dir) == 'app':
                # Chercher dans le dossier 'app'
                app_page_path = os.path.join(current_dir, 'page.html')
                return app_page_path if os.path.isfile(app_page_path) else None
            
            # Remonter d'un niveau
            parent_dir = os.path.dirname(current_dir)
            
            # Si on est à la racine et qu'on n'a pas trouvé le dossier 'app'
            if parent_dir == current_dir:
                return None
                
            current_dir = parent_dir
            
    except Exception as e:
        print(f"Erreur lors de la recherche de page.html: {e}")
        return None 


from fastapi.responses import  HTMLResponse
from jinja2 import Environment, FileSystemLoader, TemplateNotFound

def useView(data,path):
    
    env = Environment(loader=FileSystemLoader("views"))   
    
    try:
        
        template = env.get_template(path)
        return template.render(data)
    
    except TemplateNotFound:
        return HTMLResponse(content=f"Template non trouvé : {path}", status_code=404)
    except Exception as e:
        return HTMLResponse(content=f"Erreur lors du rendu du template : {str(e)}", status_code=500)




import os
from fastapi import APIRouter
from .utils import deleteFistDotte, dynamicRoute,importModule,convertPathToModulePath

# 
def FIND_ROUTES(base_path):
    routes:list = []
    
    # Verify if the 'app' folder exists
    if os.path.exists(base_path) and os.path.isdir(base_path):
        # Explore the 'app' folder and its subfolders
        for root, dirs, files in os.walk(base_path):
            #supprimers des _folder
            dirs[:] = [d for d in dirs if not d.startswith("_")]

            route = {
                "pathname": f"{'/' if os.path.basename(root) == base_path else '/' +  deleteFistDotte(os.path.relpath(root, base_path).replace("\\","/"))}",
                "dirName": root
            }
            controller = os.path.join(root, 'controller.py')

            # Check for files and add to dictionary
            if os.path.exists(controller):
                route["module"] = convertPathToModulePath(f"{root}/controller")    

            routes.append(route)

    return routes



def Router():
    """
    Charge dynamiquement les routes à partir du répertoire 'app'.
    """
    app = APIRouter()
    # Parcours des répertoires dans 'app'
    routes = FIND_ROUTES(base_path="app");
    HTTP_METHODES:tuple = ("GET", "POST", "PUT", "DELETE", "PATCH", "OPTIONS");
    for route in routes:

        pathname = dynamicRoute(route_in=route["pathname"]);

        if "module" in route:

            module = importModule(path=route["module"]);
            for attr_name in dir(module):
                attr = getattr(module, attr_name)
                
                # Vérifie que l'attribut est une fonction utilisable par FastAPI
                if callable(attr) and hasattr(attr, "__annotations__"):
                    # Ajout de la route pour chaque méthode HTTP
                    if attr_name in HTTP_METHODES:
                        
                        method = attr_name
                        app.add_api_route(pathname, attr, methods=[method])

                    # Ajout d'une route WebSocket si la méthode 'Socket' existe
                    if attr_name == "Socket":
                        app.add_api_websocket_route(f"{pathname}/ws", attr)
    return app





# def Router():
#     """
#     Charge dynamiquement les routes à partir du répertoire 'app'.
#     """
#     app = APIRouter()
#     routes = FIND_ROUTES(base_path="app")
#     HTTP_METHODES: tuple = ["DELETE", "GET", "OPTIONS", "PATCH", "POST", "PUT"]

#     for route in routes:
#         pathname = dynamicRoute(route_in=route["pathname"])

#         if "module" in route:
#             try:
#                 # Tentative d'importation du module
#                 module = importModule(path=route["module"])
                
#                 for function_name in dir(module):
#                     function = getattr(module, function_name)

#                     # Vérifie que la fonction est callable et a des annotations
#                     if callable(function) and hasattr(function, "__annotations__"):
#                         params = getattr(function, "params", {})

#                         # Ajout de la route pour chaque méthode HTTP
#                         if function_name in HTTP_METHODES:
#                             try:
#                                 app.add_api_route(
#                                     path=pathname,
#                                     endpoint=function,
#                                     methods=[function_name],
#                                     **{key: value for key, value in params.items() if key != "tags"},
#                                     tags=params.get("tags") if params.get("tags") else [pathname]
#                                 )
#                             except Exception as e:
#                                 # Capture les erreurs spécifiques à l'ajout de la route
#                                 print(f"Erreur lors de l'ajout de la route {pathname} pour la méthode {function_name}: {e}")
#                                 app.add_api_route(
#                                     path=pathname,
#                                     endpoint=lambda: {"error": f"Erreur lors de la méthode {function_name} pour la route {pathname}. {str(e)}"},
#                                     methods=[function_name],
#                                     status_code=500
#                                 )

#                         # Ajout d'une route WebSocket si la méthode 'Socket' existe
#                         elif function_name == "SOCKET":
#                             try:
#                                 app.add_api_websocket_route(f"{pathname}/ws", function)
#                             except Exception as e:
#                                 print(f"Erreur lors de l'ajout du WebSocket pour la route {pathname}: {e}")
#                                 app.add_api_websocket_route(
#                                     f"{pathname}/ws",
#                                     lambda: {"error": f"Erreur lors de la connexion WebSocket pour la route {pathname}. {str(e)}"}
#                                 )

#             except Exception as e:
#                 # Capture les erreurs d'importation ou d'autres erreurs générales
#                 print(f"Erreur lors du chargement du module {route['module']}: {e}")
#                 app.add_api_route(
#                     path=pathname,
#                     endpoint=lambda: {"error": f"Erreur lors du chargement du module {route['module']}. {str(e)}"},
#                     methods=["GET"],
#                     status_code=500
#                 )
#     return app